package types

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// PluginDevice plugin device
// swagger:model PluginDevice
type PluginDevice struct {

	// description
	// Required: true
	Description string `json:"Description"`

	// name
	// Required: true
	Name string `json:"Name"`

	// path
	// Required: true
	Path *string `json:"Path"`

	// settable
	// Required: true
	Settable []string `json:"Settable"`
}
